@@ -40,7 +40,7 @@ create_button (const gchar *label, GtkSignalFunc func)
     button = gtk_button_new_with_label (label);
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
     if (func != NULL)
-        gtk_signal_connect (GTK_OBJECT (button), "clicked",
+        g_signal_connect (G_OBJECT (button), "clicked",
           func, NULL);
     gtk_widget_show (button);
     return button;
@@ -59,10 +59,14 @@ create_chartable (void)
     {
         for (h = 0; h <= 23; h++)
         {
-            gchar *s = (gchar *) g_new0 (gchar, 5);
-            s = g_strdup_printf ("%c", v * 24 + h + 32);
-
-            button = gtk_button_new_with_label (s);
+	    char buf[7];
+            int n;
+            int ch = v * 24 + h + 32;
+	   
+	    n = g_unichar_to_utf8 (ch, buf);
+            buf[n] = 0;
+            
+            button = gtk_button_new_with_label (buf);
             mainapp->buttons = g_list_append (mainapp->buttons, button);
             gtk_widget_set_style (GTK_BIN (button)->child, mainapp->btnstyle);
             gtk_table_attach (GTK_TABLE (chartable), button, h, h + 1, v, v + 1,
@@ -70,14 +74,20 @@ create_chartable (void)
               (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
               0, 0);
 
-            gtk_signal_connect (GTK_OBJECT (button), "clicked",
-              GTK_SIGNAL_FUNC (cb_charbtn_click), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "enter",
+            g_signal_connect (G_OBJECT (button), "clicked",
+              G_CALLBACK (cb_charbtn_click), NULL);
+            g_signal_connect (G_OBJECT (button), "enter",
+              G_CALLBACK (cb_charbtn_enter), NULL);
+            g_signal_connect (G_OBJECT (button), "leave",
+              G_CALLBACK (cb_charbtn_leave), NULL);
+
+
+            g_signal_connect (G_OBJECT (button), "focus_in_event",
               GTK_SIGNAL_FUNC (cb_charbtn_enter), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "leave",
+            g_signal_connect (G_OBJECT (button), "focus_out_event",
               GTK_SIGNAL_FUNC (cb_charbtn_leave), NULL);
 
-            g_free (s);
+
         }
     }
 
@@ -85,10 +95,17 @@ create_chartable (void)
     {
         for (h = 0; h <= 23; h++)
         {
-            gchar *s = (gchar *) g_new0 (gchar, 5);
-            s = g_strdup_printf ("%c", v * 24 + h + 161);
+            char buf[7];
+            int n;
+	    int ch = v * 24 + h + 161;
+	    
+	    if (ch > 0xff)
+		    continue;
 
-            button = gtk_button_new_with_label (s);
+            n = g_unichar_to_utf8 (ch, buf);
+            buf[n] = 0;
+	    
+            button = gtk_button_new_with_label (buf);
             mainapp->buttons = g_list_append (mainapp->buttons, button);
             gtk_widget_set_style (GTK_BIN (button)->child, mainapp->btnstyle);
             gtk_table_attach (GTK_TABLE (chartable), button,
@@ -97,44 +114,91 @@ create_chartable (void)
               (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
               0, 0);
 
-            gtk_signal_connect (GTK_OBJECT (button), "clicked",
-              GTK_SIGNAL_FUNC (cb_charbtn_click), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "enter",
+            g_signal_connect (G_OBJECT (button), "clicked",
+              G_CALLBACK (cb_charbtn_click), NULL);
+            g_signal_connect (G_OBJECT (button), "enter",
+              G_CALLBACK (cb_charbtn_enter), NULL);
+            g_signal_connect (G_OBJECT (button), "leave",
+              G_CALLBACK (cb_charbtn_leave), NULL);
+
+
+            g_signal_connect (G_OBJECT (button), "focus_in_event",
               GTK_SIGNAL_FUNC (cb_charbtn_enter), NULL);
-            gtk_signal_connect (GTK_OBJECT (button), "leave",
+            g_signal_connect (G_OBJECT (button), "focus_out_event",
               GTK_SIGNAL_FUNC (cb_charbtn_leave), NULL);
 
-            g_free (s);
+
         }
     }
 
     gtk_widget_show_all (chartable);
-    gtk_widget_push_style (mainapp->btnstyle);
-    gtk_widget_pop_style ();
     return chartable;
 }
 
+/* Check if gail is loaded */
+gboolean
+check_gail(GtkWidget *widget)
+{
+   return GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(widget));
+}
+
+
+/* Add AtkName and AtkDescription */
+void
+add_atk_namedesc(GtkWidget *widget, const gchar *name, const gchar *desc)
+{
+   AtkObject *atk_widget;
+   atk_widget = gtk_widget_get_accessible(widget);
+   atk_object_set_name(atk_widget, name);
+   atk_object_set_description(atk_widget, desc);
+}
+
+
+/* Add AtkRelation */
+void
+add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType type)
+{
+
+    AtkObject *atk_obj1, *atk_obj2;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+
+    atk_obj1 = gtk_widget_get_accessible(obj1);
+
+    atk_obj2 = gtk_widget_get_accessible(obj2);
+
+    relation_set = atk_object_ref_relation_set (atk_obj1);
+    relation = atk_relation_new(&atk_obj2, 1, type);
+    atk_relation_set_add(relation_set, relation);
+    g_object_unref(G_OBJECT (relation));
+
+}
+
 
 static void
 main_app_create_ui (MainApp *app)
 {
     GtkWidget *appbar;
-    GtkWidget *vbox, *hbox, *hbox2, *vbox2;
-    GtkWidget *vsep, *alabel;
+    GtkWidget *vbox, *hbox, *hbox2, *hbox3, *vbox2;
+    GtkWidget *vsep, *alabel, *label;
     GtkWidget *chartable;
     GtkWidget *buttonbox, *button;
+    GtkWidget *align;
     GtkWidget *viewport;
+    GtkWidget *image;
+
 
     /* Main window */
     {
-        GnomeDockLayoutItem *item;
+        BonoboDockLayoutItem *item;
 
         app->window = gnome_app_new (_(PACKAGE), _("Gnome Character Map"));
         gtk_widget_set_name (app->window, "mainapp");
-        gtk_signal_connect_object (GTK_OBJECT (app->window), "destroy",
-          GTK_SIGNAL_FUNC (main_app_destroy), GTK_OBJECT (app));
+        
+        g_signal_connect_swapped (G_OBJECT (app->window), "destroy",
+          G_CALLBACK (main_app_destroy), G_OBJECT (app));
         gtk_widget_realize (app->window);
-
+	
         appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
         gnome_app_set_statusbar (GNOME_APP (app->window), appbar);
         gtk_widget_show (appbar);
@@ -142,9 +206,7 @@ main_app_create_ui (MainApp *app)
         gnome_app_create_menus (GNOME_APP (app->window), menubar);
         gnome_app_install_menu_hints (GNOME_APP (app->window), menubar);
 
-        gnome_app_create_toolbar (GNOME_APP (app->window), toolbar);
         item = g_list_nth_data (GNOME_APP (app->window)->layout->items, 0);
-        app->actionbar = GTK_WIDGET (item->item);
     }
 
     /* The toplevel vbox */
@@ -154,37 +216,55 @@ main_app_create_ui (MainApp *app)
       app->window)->contents), 8);
 
     {
-        GnomeDockLayoutItem *item;
+        BonoboDockLayoutItem *item;
 
         hbox = gtk_hbox_new (FALSE, 6);
         gtk_container_set_border_width (GTK_CONTAINER (hbox), 1);
         gnome_app_add_docked (GNOME_APP (app->window), hbox, _("Action Toolbar"),
-          GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL,
-          GNOME_DOCK_TOP, 2, 0, 1);
+          BONOBO_DOCK_ITEM_BEH_EXCLUSIVE | BONOBO_DOCK_ITEM_BEH_NEVER_VERTICAL,
+          BONOBO_DOCK_TOP, 2, 0, 1);
 
-        app->fontpicker = gnome_font_picker_new ();
-        gnome_font_picker_set_mode (GNOME_FONT_PICKER (app->fontpicker),
-          GNOME_FONT_PICKER_MODE_FONT_INFO);
-        gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (app->fontpicker),
-          FALSE, 12);
-        gnome_font_picker_set_font_name (GNOME_FONT_PICKER (app->fontpicker),
-          "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-*-*");
-        gtk_button_set_relief (GTK_BUTTON (app->fontpicker), GTK_RELIEF_NONE);
-        gtk_box_pack_start (GTK_BOX (hbox), app->fontpicker, FALSE, TRUE, 0);
-        gtk_signal_connect (GTK_OBJECT (app->fontpicker), "font_set",
-          GTK_SIGNAL_FUNC (cb_fontpicker_font_set), NULL);
-
-        vsep = gtk_vseparator_new ();
-        gtk_box_pack_start (GTK_BOX (hbox), vsep, FALSE, FALSE, 0);
-
-        alabel = gtk_label_new (_("Text to copy:"));
+        alabel = gtk_label_new_with_mnemonic (_("_Text to copy:"));
+        gtk_misc_set_padding (GTK_MISC (alabel), GNOME_PAD_SMALL, -1);
         gtk_box_pack_start (GTK_BOX (hbox), alabel, FALSE, TRUE, 0);
 
         app->entry = gtk_entry_new ();
+        gtk_label_set_mnemonic_widget (GTK_LABEL (alabel), app->entry);
         gtk_box_pack_start (GTK_BOX (hbox), app->entry, TRUE, TRUE, 0);
-        gnome_popup_menu_attach (gnome_popup_menu_new (edit_menu),
-          app->entry, NULL);
+        
+	button = gtk_button_new ();
+	if (GTK_BIN (button)->child)
+                gtk_container_remove (GTK_CONTAINER (button),
+                                      GTK_BIN (button)->child);
 
+	label = gtk_label_new_with_mnemonic (_("_Copy"));
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+        image = gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_BUTTON);
+        hbox3 = gtk_hbox_new (FALSE, 2);
+
+        align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+        gtk_box_pack_start (GTK_BOX (hbox3), image, FALSE, FALSE, 0);
+        gtk_box_pack_end (GTK_BOX (hbox3), label, FALSE, FALSE, 0);
+
+        gtk_container_add (GTK_CONTAINER (button), align);
+        gtk_container_add (GTK_CONTAINER (align), hbox3);
+	
+        gtk_widget_show_all (align);
+
+        gtk_container_set_border_width (GTK_CONTAINER (button), 2);
+        g_signal_connect (G_OBJECT (button), "clicked",
+        		  G_CALLBACK (cb_copy_click), NULL);
+        gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);		 
+        if (check_gail(app->entry))
+        {
+          add_atk_namedesc(GTK_WIDGET(alabel), _("Text to copy"), _("Text to copy"));
+          add_atk_namedesc(app->entry, _("Entry"), _("Text to copy"));
+          add_atk_namedesc(button, _("Copy"), _("Copy the text"));
+          add_atk_relation(app->entry, GTK_WIDGET(alabel), ATK_RELATION_LABELLED_BY);
+        }
+ 
         gtk_widget_show_all (hbox);
         item = g_list_nth_data (GNOME_APP (app->window)->layout->items, 0);
         app->textbar = GTK_WIDGET (item->item);
@@ -197,13 +277,12 @@ main_app_create_ui (MainApp *app)
     /* The character table */
     {
         GtkWidget *tmp;
+	GdkFont *font;
 
         tmp = gtk_button_new ();
         gtk_widget_ensure_style(tmp);
         app->btnstyle = gtk_style_copy (gtk_widget_get_style (tmp));
-        app->btnstyle->font = gdk_fontset_load (
-          "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-*-*"
-        );
+
         gtk_widget_destroy (tmp);
 
         chartable = create_chartable ();
@@ -214,64 +293,9 @@ main_app_create_ui (MainApp *app)
     vbox2 = gtk_vbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox2), vbox2, FALSE, TRUE, 0);
 
-
-    /* The buttonbox */
-    {
-        buttonbox = gtk_vbutton_box_new ();
-        gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_START);
-        gtk_button_box_set_spacing (GTK_BUTTON_BOX (buttonbox), 0);
-        gtk_box_pack_start (GTK_BOX (vbox2), buttonbox, TRUE, TRUE, 0);
-
-        gtk_container_add (GTK_CONTAINER (buttonbox),
-          create_button (_("Exit"), GTK_SIGNAL_FUNC (cb_exit_click)));
-        button = create_button (_("Copy"), GTK_SIGNAL_FUNC (cb_copy_click));
-        gtk_container_add (GTK_CONTAINER (buttonbox), button);
-        gtk_widget_grab_default (button);
-        gtk_container_add (GTK_CONTAINER (buttonbox),
-          create_button (_("About"), GTK_SIGNAL_FUNC (cb_about_click)));
-        gtk_container_add (GTK_CONTAINER (buttonbox),
-          create_button (_("Help"), GTK_SIGNAL_FUNC (cb_help_click)));
-    }
-
-    /* The zoom viewport */
-    {
-        GtkStyle *style;
-        GdkColor black = {0, 0, 0, 0};
-        GdkColor white = {0, 0xFFFF, 0xFFFF, 0xFFFF};
-        guint8 i;
-
-        viewport = gtk_viewport_new (NULL, NULL);
-        gdk_color_alloc (gdk_colormap_get_system (), &black);
-        gdk_color_alloc (gdk_colormap_get_system (), &white);
-
-        style = gtk_style_copy (gtk_widget_get_style (viewport));
-        for (i = 0; i < 5; i++) style->fg[i] = white;
-        for (i = 0; i < 5; i++) style->bg[i] = black;
-        style->font = gdk_fontset_load (
-          "-*-helvetica-bold-r-normal-*-*-180-*-*-p-*-*-*,*-r-*"
-        );
-
-        gtk_widget_set_style (viewport, style);
-        gtk_box_pack_start (GTK_BOX (vbox2), viewport, FALSE, TRUE, 0);
-
-        app->preview_label = gtk_label_new (NULL);
-        gtk_container_add (GTK_CONTAINER (viewport), app->preview_label);
-        gtk_widget_set_style (app->preview_label, style);
-
-        gtk_widget_push_style (style);
-        gtk_widget_pop_style ();
-    }
-
     gtk_widget_show_all (vbox);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-      view_menu[0].widget), TRUE);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-      view_menu[1].widget), TRUE);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-      view_menu[2].widget), TRUE);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-      settings_menu[3].widget), TRUE);
     gtk_widget_grab_focus (app->entry);
+    
 }
 
 
@@ -282,24 +306,34 @@ main_app_init (MainApp *obj)
     main_app_create_ui (obj);
 }
 
+static void
+main_app_class_init (MainAppClass *klass)
+{
+    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    
+}
 
-guint
+GType
 main_app_get_type (void)
 {
-    static guint ga_type = 0;
+    static GType ga_type = 0;
 
+    g_type_init ();
+    
     if (!ga_type) {
-        GtkTypeInfo ga_info = {
-          "MainApp",
-          sizeof (MainApp),
+        static const GTypeInfo ga_info = {
           sizeof (MainAppClass),
-          (GtkClassInitFunc) NULL,
-          (GtkObjectInitFunc) main_app_init,
-          (GtkArgSetFunc) NULL,
-          (GtkArgGetFunc) NULL,
-          (GtkClassInitFunc) NULL
+          (GBaseInitFunc) NULL,
+          (GBaseFinalizeFunc) NULL,
+          (GClassInitFunc) main_app_class_init,
+          NULL,
+          NULL,
+          sizeof (MainApp),
+          0,
+          (GInstanceInitFunc) main_app_init,
         };
-        ga_type = gtk_type_unique (gtk_object_get_type (), &ga_info);
+        ga_type = g_type_register_static (G_TYPE_OBJECT, "MainApp",
+        				  &ga_info, 0);
     }
     return ga_type;
 }
@@ -308,7 +342,7 @@ main_app_get_type (void)
 MainApp *
 main_app_new (void)
 {
-    return MAIN_APP (gtk_type_new ((GtkType) MAIN_APP_TYPE));
+    return MAIN_APP (g_object_new (MAIN_APP_TYPE, NULL));
 }
 
 
@@ -318,9 +352,6 @@ main_app_destroy (MainApp *obj)
     g_return_if_fail (obj != NULL);
     g_return_if_fail (MAIN_IS_APP (obj) == TRUE);
 
-    if (obj->window != NULL) gtk_widget_destroy (obj->window);
-    if (obj->btnstyle != NULL) g_free (obj->btnstyle);
-    gtk_object_destroy (GTK_OBJECT (obj));
     gtk_main_quit ();
 }
 
