@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.17.2.2 2002/06/26 14:15:35 gman Exp $ */
+/* $Id: gdict-app.c,v 1.48 2002/04/30 15:17:54 paolo Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -23,12 +23,12 @@
 #include <string.h>
 #include <ctype.h>
 
+#define GTK_ENABLE_BROKEN
 #include <gnome.h>
 
 #include "dict.h"
 #include "gdict-about.h"
 #include "gdict-pref.h"
-#include "gdict-applet.h"
 
 #include "gdict-app.h"
 #include "gdict-defbox.h"
@@ -38,7 +38,7 @@
 #  include <libgnomeprint/gnome-printer-dialog.h>
 #endif /* HAVE_GNOME_PRINT */
 
-#define APPNAME "gdict"
+#define APPNAME "gnome-dictionary"
 
 GtkWidget *gdict_app;
 GtkWidget *gdict_appbar;
@@ -48,6 +48,7 @@ GDictDefbox *defbox;
 GDictSpeller *speller;
 GtkWidget *pref_dialog;
 GtkWidget *socket_error_dialog;
+gboolean  gail_loaded = FALSE;
 
 #ifdef HAVE_GNOME_PRINT
 GnomePrinter *gdict_printer = NULL;
@@ -65,12 +66,12 @@ socket_dialog_close_cb (GtkWidget *widget, gpointer data)
 static void
 socket_error_cb (GtkWidget *widget, gchar *message, gpointer data) 
 {
-    if (!socket_error_dialog) {
-        socket_error_dialog = 
-            gnome_error_dialog_parented (message, GTK_WINDOW (gdict_app));
-        gtk_signal_connect (GTK_OBJECT (socket_error_dialog), "close",
-                            GTK_SIGNAL_FUNC (socket_dialog_close_cb), NULL);
-    }
+    socket_error_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  		  GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                  		  "%s", message, NULL); 
+    gtk_dialog_run (GTK_DIALOG (socket_error_dialog));
+    gtk_widget_destroy (socket_error_dialog);
+    
 }
 
 /* gdict_init_context
@@ -88,6 +89,8 @@ gdict_init_context (void)
     
     context = dict_context_new (gdict_pref.server, gdict_pref.port);
     context->command = dict_disconnect_command_new ();
+    
+    defbox->context = context;
 
     if (context->hostinfo)
 	return 0;
@@ -101,7 +104,7 @@ gdict_app_clear (void)
     gchar *word;
     
     gdict_defbox_clear (defbox);
-    
+  
     /* Update entry */
     if ((word = gdict_defbox_get_word (defbox))) {
         gtk_entry_set_text (GTK_ENTRY(word_entry), word);
@@ -133,20 +136,11 @@ spell_not_found_cb (GtkWidget *widget, gpointer data)
     gnome_appbar_refresh (GNOME_APPBAR (gdict_appbar));
 }
 
-static gint
-spell_close_cb (GtkWidget *widget, gpointer data) 
-{
-    speller = NULL;
-    return FALSE;
-}
-
 void
 gdict_open_speller (void) 
 {
     if (!speller) {
         speller = GDICT_SPELLER (gdict_speller_new (context));
-        gnome_dialog_set_parent (GNOME_DIALOG (speller), 
-                                 GTK_WINDOW (gdict_app));
         
         if (!speller) return;
 
@@ -158,11 +152,10 @@ gdict_open_speller (void)
                             GTK_SIGNAL_FUNC (spell_not_found_cb), NULL);
         gtk_signal_connect (GTK_OBJECT (speller), "socket_error",
                             GTK_SIGNAL_FUNC (socket_error_cb), NULL);
-        gtk_signal_connect (GTK_OBJECT (speller), "close",
-                            GTK_SIGNAL_FUNC (spell_close_cb), NULL);
+        gtk_widget_show_all (GTK_WIDGET (speller));
     }
-    
-    gtk_widget_show (GTK_WIDGET (speller));
+    else
+    	gtk_window_present (GTK_WINDOW (speller));
 }
 
 
@@ -215,9 +208,11 @@ gdict_app_do_lookup (gchar *text)
 static void
 lookup_entry (void) 
 {
-    gchar *text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    gchar *text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     g_strdown(text);
     gdict_app_do_lookup (text);
+    if (text)
+        g_free (text);
 }
 
 static void
@@ -227,19 +222,23 @@ lookup_defbox (void)
     gtk_signal_emit_by_name (GTK_OBJECT (defbox), "copy_clipboard");
     gtk_entry_set_text (GTK_ENTRY (word_entry), "");
     gtk_signal_emit_by_name (GTK_OBJECT (word_entry), "paste_clipboard");
-    text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     g_strdown (text);
     gnome_entry_append_history (GNOME_ENTRY (gnome_word_entry), 1, text);
     gdict_app_do_lookup (text);
+    if (text)
+        g_free (text);
 }
 
 static void
 lookup_any (void) 
 {
-    if (GTK_WIDGET_HAS_FOCUS (word_entry))
-	lookup_entry ();
-    else if (GTK_WIDGET_HAS_FOCUS (defbox))
+    /*if (GTK_WIDGET_HAS_FOCUS (word_entry))
+	lookup_entry ();*/
+    if (GTK_WIDGET_HAS_FOCUS (defbox))
 	lookup_defbox ();
+    else
+        lookup_entry ();
 }
 
 static void
@@ -284,14 +283,39 @@ lookup_cb (GtkWidget *menuitem, gpointer user_data)
     lookup_any();
 }
 
+static void
+lookup_button_cb (GtkButton *button, gpointer data)
+{
+    lookup_any();
+}
+
+static void
+lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+		       GtkSelectionData *sd, guint info, guint t, gpointer data)
+{
+	gchar *text;
+	
+	text = gtk_selection_data_get_text (sd);
+	
+	if (text) {
+		g_strdown (text);
+		gtk_entry_set_text (GTK_ENTRY (word_entry), text);
+    		gnome_entry_append_history (GNOME_ENTRY (gnome_word_entry), 1, text);
+    		gdict_app_do_lookup (text);
+		g_free (text);
+	}
+}
+
 static void
 spell_cb (GtkWidget *menuitem, gpointer user_data) 
 {
     gchar *text;
 
-    text = gtk_entry_get_text (GTK_ENTRY (word_entry));
+    text = gtk_editable_get_chars (GTK_EDITABLE (word_entry), 0, -1);
     if (gdict_spell (text, FALSE) < 0)
 	gdict_not_online ();
+    if (text)
+        g_free (text);
 }
 
 #ifdef HAVE_GNOME_PRINT
@@ -346,14 +370,24 @@ paste_cb (GtkWidget *menuitem, gpointer user_data)
 static void
 clear_cb (GtkWidget *menuitem, gpointer user_data) 
 {
-    gtk_editable_select_region (GTK_EDITABLE (defbox), 0, 0);
+    gdict_defbox_clear (defbox);
 }
 
 static void
 select_all_cb (GtkMenuItem *menuitem, gpointer user_data) 
 {
-    gtk_editable_select_region (GTK_EDITABLE (defbox), 0, 
-                                gtk_text_get_length (GTK_TEXT (defbox)));
+    GtkTextBuffer* buffer = NULL;
+    GtkTextIter start_iter, end_iter;
+    
+    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
+    
+    gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
+    
+    gtk_text_buffer_place_cursor (buffer, &end_iter);
+    gtk_text_buffer_move_mark (buffer,
+                               gtk_text_buffer_get_mark (buffer, "selection_bound"),
+                               &start_iter);
+
 }
 
 static gchar *find_text;
@@ -361,13 +395,17 @@ static gchar *find_text;
 static void
 prompt_response_cb (GnomeAppBar *ab) 
 {
+    gchar *tmp;
+
+    tmp = gnome_appbar_get_response(ab);
     /* fprintf(stderr, "Response\n"); */
-    if (ab->prompt) {
+    if (tmp) {
         g_free(find_text);
         find_text = gnome_appbar_get_response(ab);
         if (find_text)
             gdict_defbox_find (defbox, find_text, TRUE);
         gnome_appbar_clear_prompt(GNOME_APPBAR(gdict_appbar));
+	g_free(tmp);
     }
 }
 
@@ -376,7 +414,7 @@ find_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
     if (!defbox->def_cmd)
         return;
-
+    gnome_appbar_pop (GNOME_APPBAR (gdict_appbar));
     gnome_appbar_set_prompt (GNOME_APPBAR (gdict_appbar), _("Find:"), TRUE);
 }
 
@@ -389,13 +427,6 @@ find_again_cb (GtkMenuItem *menuitem, gpointer user_data)
         gdict_defbox_find (defbox, find_text, FALSE);
 }
 
-static gint
-pref_dialog_close_cb (GtkWidget *widget, gpointer user_data) 
-{
-    pref_dialog = NULL;
-    return FALSE;
-}
-
 static void
 pref_dialog_apply_cb (GtkWidget *widget, gpointer user_data) 
 {
@@ -409,28 +440,23 @@ gdict_not_online ()
     GtkWidget *w;
     gchar *s;
     
-    s = g_strdup_printf (_("Unable to perform requested operation, either because the server\n"
-			   "you are using is down or because you are not connected to the\n"
-			   "Internet. If you are not already connected to the Internet, please\n" 
-    			   "do so and try again."));
-    w = gnome_error_dialog (s);
-    gnome_dialog_run_and_close (GNOME_DIALOG (w));
+    s = g_strdup_printf (_("Unable to perform requested operation. \n"
+    		           "Either the server you are using is not available \n"
+    		           "or you are not connected to the Internet."));
+    w = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                     "%s", s, NULL); 
+    gtk_dialog_run (GTK_DIALOG (w));
+    gtk_widget_destroy (w);
     g_free (s);
 }
 
 void
 gdict_app_show_preferences (void) 
 {
-    if (!pref_dialog) {
+	
+    if (!pref_dialog)
         pref_dialog = gdict_pref_dialog_new (context);
-        gtk_signal_connect (GTK_OBJECT (pref_dialog), "apply_changes",
-                            GTK_SIGNAL_FUNC (pref_dialog_apply_cb), NULL);
-        gtk_signal_connect (GTK_OBJECT (pref_dialog), "socket_error",
-                            GTK_SIGNAL_FUNC (socket_error_cb), NULL);
-        gtk_signal_connect (GTK_OBJECT (pref_dialog), "close",
-                            GTK_SIGNAL_FUNC (pref_dialog_close_cb),
-			    NULL);
-    }
 
     gtk_widget_show (pref_dialog);
 }
@@ -447,11 +473,116 @@ about_cb (GtkMenuItem *menuitem, gpointer user_data)
     gdict_about();
 }
 
-static GnomeUIInfo file_menu_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("Look up"),
-			    N_("Look up word in dictionary"),
+
+/*
+ * add_atk_namedesc
+ * @widget    : The Gtk Widget for which @name and @desc are added.
+ * @name      : Accessible Name
+ * @desc      : Accessible Description
+ * Description: This function adds accessible name and description to a
+ *              Gtk widget.
+ */
+
+void
+add_atk_namedesc(GtkWidget *widget, const gchar *name, const gchar *desc)
+{
+    AtkObject *atk_widget;
+
+    g_return_if_fail (GTK_IS_WIDGET(widget));
+
+    atk_widget = gtk_widget_get_accessible(widget);
+    
+    if( name != NULL )
+    atk_object_set_name(atk_widget, name);
+    if( desc !=NULL )
+    atk_object_set_description(atk_widget, desc);
+}
+
+
+/*
+ * add_atk_relation
+ * @obj1      : The first widget in the relation @rel_type
+ * @obj2      : The second widget in the relation @rel_type.
+ * @rel_type  : Relation type which relates @obj1 and @obj2
+ * Description: This function establishes Atk Relation between two given
+ *              objects.
+ */
+
+void
+add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType rel_type)
+{
+
+    AtkObject *atk_obj1, *atk_obj2;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+
+    g_return_if_fail (GTK_IS_WIDGET(obj1));
+    g_return_if_fail (GTK_IS_WIDGET(obj2));
+
+    atk_obj1 = gtk_widget_get_accessible(obj1);
+
+    atk_obj2 = gtk_widget_get_accessible(obj2);
+
+    relation_set = atk_object_ref_relation_set (atk_obj1);
+    relation = atk_relation_new(&atk_obj2, 1, rel_type);
+    atk_relation_set_add(relation_set, relation);
+    g_object_unref(G_OBJECT (relation));
+
+}
+
+GtkWidget* 
+gdict_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+ 	if (GTK_BIN (button)->child)
+    		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+  	if (gtk_stock_lookup (stock_id, &item))
+    	{
+      		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+      
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+      		hbox = gtk_hbox_new (FALSE, 2);
+
+      		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+      
+      		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+      		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+      
+      		gtk_container_add (GTK_CONTAINER (button), align);
+      		gtk_container_add (GTK_CONTAINER (align), hbox);
+      		gtk_widget_show_all (align);
+
+      		return button;
+    	}
+
+      	label = gtk_label_new_with_mnemonic (text);
+      	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+  
+  	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+  	gtk_widget_show (label);
+  	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+static GnomeUIInfo dictionary_menu_uiinfo[] = {
+    GNOMEUIINFO_ITEM_STOCK (N_("_Look Up Word"),
+			    N_("Lookup word in dictionary"),
 			    lookup_cb, GNOME_STOCK_MENU_SEARCH),
-    GNOMEUIINFO_ITEM_STOCK (N_("Spell"), 
+    GNOMEUIINFO_ITEM_STOCK (N_("Check _Spelling"), 
 			    N_("Check word spelling"), 
 			    spell_cb, GNOME_STOCK_MENU_SPELLCHECK),
 #ifdef HAVE_GNOME_PRINT
@@ -460,7 +591,7 @@ static GnomeUIInfo file_menu_uiinfo[] = {
     GNOMEUIINFO_MENU_PRINT_SETUP_ITEM (print_setup_cb, NULL),
 #endif /* HAVE_GNOME_PRINT */
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_MENU_EXIT_ITEM (exit_cb, NULL),
+    GNOMEUIINFO_MENU_QUIT_ITEM(exit_cb, NULL),
     GNOMEUIINFO_END
 };
 
@@ -482,14 +613,13 @@ static GnomeUIInfo edit_menu_uiinfo[] = {
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_MENU_FIND_ITEM (find_cb, NULL),
     GNOMEUIINFO_MENU_FIND_AGAIN_ITEM (find_again_cb, NULL),
-    GNOMEUIINFO_END
-};
-
-static GnomeUIInfo settings_menu_uiinfo[] = {
+    GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
     GNOMEUIINFO_END
 };
 
+
+
 static GnomeUIInfo help_menu_uiinfo[] = {
     GNOMEUIINFO_HELP(APPNAME),
     GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
@@ -497,79 +627,109 @@ static GnomeUIInfo help_menu_uiinfo[] = {
 };
 
 static GnomeUIInfo menubar_uiinfo[] = {
-    GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
+    { GNOME_APP_UI_SUBTREE_STOCK, N_("_Dictionary"), NULL, 
+      dictionary_menu_uiinfo, NULL, NULL, (GnomeUIPixmapType) 0, 
+      NULL, 0, (GdkModifierType) 0, NULL },
     GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
-    GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu_uiinfo),
     GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
     GNOMEUIINFO_END
 };
 
-static GnomeUIInfo toolbar_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("Look up"), 
-			    N_("Look up word in dictionary"), 
-			    lookup_cb, GNOME_STOCK_PIXMAP_SEARCH),
-    GNOMEUIINFO_ITEM_STOCK (N_("Spell"), 
-			    N_("Check word spelling"), 
-			    spell_cb, GNOME_STOCK_PIXMAP_SPELLCHECK),
-    GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK (N_("Cut"), 
-			    N_("Cut to Clipboard"), 
-			    cut_cb, GNOME_STOCK_PIXMAP_CUT),
-    GNOMEUIINFO_ITEM_STOCK (N_("Copy"), 
-			    N_("Copy to Clipboard"), 
-			    copy_cb, GNOME_STOCK_PIXMAP_COPY),
-    GNOMEUIINFO_ITEM_STOCK (N_("Paste"), 
-			    N_("Paste Clipboard Contents"), 
-			    paste_cb, GNOME_STOCK_PIXMAP_PASTE),
-    GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK (N_("Settings"), 
-			  N_("Server to use, default search strategy, fonts"), 
-			    preferences_cb, GNOME_STOCK_PIXMAP_PREFERENCES),
-    GNOMEUIINFO_END
+enum
+{
+  TARGET_STRING,
+  TARGET_TEXT,
+  TARGET_COMPOUND_TEXT,
+  TARGET_UTF8_STRING,
+  TARGET_TEXT_BUFFER_CONTENTS
 };
 
-GtkWidget *gdict_app_create (void) {
+GtkWidget *gdict_app_create (gboolean applet) {
     GtkWidget *dock;
-    GtkWidget *vbox, *hbox;
-    GtkAdjustment *vscrolladj;
+    GtkWidget *vbox;
+    GtkWidget *hbox;
+    GtkWidget *button;
+    GtkWidget *label, *word_label;
+    GtkWidget *scrolled;
     GtkWidget *defbox_vscrollbar;
     GtkTooltips *tooltips;
+    static GtkTargetEntry drop_targets [] = {
+    	{ "UTF8_STRING", 0, 0 },
+  	{ "COMPOUND_TEXT", 0, 0 },
+  	{ "TEXT", 0, 0 },
+  	{ "text/plain", 0, 0 },
+  	{ "STRING",     0, 0 }
+    };
   
     tooltips = gtk_tooltips_new ();
   
-    gdict_app = gnome_app_new ("GDict", "GDict");
-    gtk_widget_set_usize (gdict_app, 600, 330);
-  
+    gdict_app = gnome_app_new ("gnome-dictionary", _("Dictionary"));
+    gtk_window_set_default_size (GTK_WINDOW (gdict_app), 450, 330);
+    
     dock = GNOME_APP (gdict_app)->dock;
     gtk_widget_show (dock);
-
-    if (gdict_applet_toggle)  /* quick hack */
-        memcpy (&file_menu_uiinfo[EXIT_FILE_MENU_ITEM],
+    
+    if (applet)  /* quick hack */
+        memcpy (&dictionary_menu_uiinfo[EXIT_FILE_MENU_ITEM],
 		&applet_close_item, sizeof(GnomeUIInfo));
 
     gnome_app_create_menus (GNOME_APP (gdict_app), menubar_uiinfo);
-    gnome_app_create_toolbar (GNOME_APP (gdict_app), toolbar_uiinfo);
     
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
     gnome_app_set_contents (GNOME_APP (gdict_app), vbox);
+    
+    hbox = gtk_hbox_new (FALSE, 6);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    
+    word_label = gtk_label_new_with_mnemonic (_("_Word: "));
+    gtk_widget_show ( GTK_WIDGET(word_label) );
+    gtk_box_pack_start (GTK_BOX (hbox), word_label, FALSE, FALSE, 0);
+    
+    button = gdict_button_new_with_stock_image (_("_Look Up Word"), GTK_STOCK_FIND);
+    gtk_widget_show (button);
+    gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_drag_dest_set (button, GTK_DEST_DEFAULT_ALL, drop_targets, 
+    		       G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
+    g_signal_connect (G_OBJECT (button), "clicked",
+    		      G_CALLBACK (lookup_button_cb), defbox);
+    g_signal_connect (G_OBJECT (button), "drag_data_received",
+    		      G_CALLBACK (lookup_button_drag_cb), defbox);
 
-    gnome_word_entry = gnome_entry_new ("Word entry");
+    gnome_word_entry = gnome_entry_new ("Wordentry");
     gtk_widget_show (gnome_word_entry);
-    gtk_box_pack_start (GTK_BOX (vbox), gnome_word_entry, FALSE, FALSE, 0);
-    GTK_WIDGET_SET_FLAGS (gnome_word_entry, GTK_CAN_FOCUS);
-    gtk_widget_grab_focus (gnome_word_entry);
+    gtk_box_pack_start (GTK_BOX (hbox), gnome_word_entry, TRUE, TRUE, 0);
     gnome_entry_set_max_saved (GNOME_ENTRY (gnome_word_entry), 50);
     word_entry = gnome_entry_gtk_entry(GNOME_ENTRY (gnome_word_entry));
-    hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_show (hbox);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+   
+    gtk_label_set_mnemonic_widget (GTK_LABEL (word_label), word_entry);
+    GTK_WIDGET_SET_FLAGS (word_entry, GTK_CAN_FOCUS);
+    gtk_widget_grab_focus (word_entry);
 
-    vscrolladj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 0, 0, 0, 0));
-    defbox_vscrollbar = gtk_vscrollbar_new (vscrolladj);
-    gtk_widget_show (defbox_vscrollbar);
+    if ( GTK_IS_ACCESSIBLE (gtk_widget_get_accessible(gnome_word_entry)))
+    {
+        gail_loaded = TRUE;
+        add_atk_namedesc(GTK_WIDGET(word_label), _("Word"), _("Word"));
+        add_atk_namedesc(gnome_word_entry, _("Word Entry"), _("Enter a Word or select one from the list below"));
+        add_atk_namedesc(word_entry, _("Word Entry"), NULL);
+        add_atk_relation(gnome_word_entry, GTK_WIDGET(word_label), ATK_RELATION_LABELLED_BY);
+        add_atk_namedesc(GTK_WIDGET(button), NULL, _("Look Up for a Word"));
+    }
+
+    scrolled = gtk_scrolled_window_new (NULL, NULL);
+    gtk_container_set_border_width (GTK_CONTAINER (scrolled), GNOME_PAD_SMALL);    
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+                                        GTK_POLICY_AUTOMATIC,             
+	       				GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled), GTK_SHADOW_ETCHED_IN);
+
+    gtk_widget_show (scrolled);
   
     defbox = GDICT_DEFBOX (gdict_defbox_new ());
+    defbox_setup_tags (defbox);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW (defbox), GNOME_PAD_SMALL);
     gtk_signal_connect (GTK_OBJECT (defbox), "word_lookup_start",
                         GTK_SIGNAL_FUNC (def_lookup_start_cb), defbox);
     gtk_signal_connect (GTK_OBJECT (defbox), "word_lookup_done",
@@ -581,18 +741,17 @@ GtkWidget *gdict_app_create (void) {
     gtk_signal_connect (GTK_OBJECT (defbox), "socket_error",
                         GTK_SIGNAL_FUNC (socket_error_cb), defbox);
   
-    gtk_text_set_adjustments (GTK_TEXT (defbox), NULL, vscrolladj);
     gtk_widget_show (GTK_WIDGET (defbox));
-    gtk_tooltips_set_tip (tooltips, GTK_WIDGET (defbox), 
-                          _("Word definition"), NULL);
-    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (defbox), TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), defbox_vscrollbar, FALSE, TRUE, 0);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (defbox), FALSE);
+    gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (defbox));
 
-    gdict_appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_ALWAYS);
+    gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
+
+    gdict_appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_ALWAYS);
     gtk_widget_show (gdict_appbar);
     gnome_app_set_statusbar (GNOME_APP (gdict_app), gdict_appbar);
-    
-    if (!gdict_applet_toggle)
+
+    if (!applet)
       gtk_signal_connect (GTK_OBJECT (gdict_app), "delete_event",
                           GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
     else
