@@ -72,6 +72,7 @@ extern UserPrefsStruct *user_prefs;
 
 GdkGC *gc;
 GdkDrawable *canvas;
+PangoLayout *log_layout;
 
 Log *loglist[MAX_NUM_LOGS];
 Log *curlog;
@@ -79,14 +80,12 @@ int numlogs, curlognum;
 int log_line_sep;
 int cursor_visible;
 int canvas_width, canvas_height;
-char *deflognames[] =
-{PATH_MESSAGES};
-
 char *month[12] =
 {N_("January"), N_("February"), N_("March"), N_("April"), N_("May"),
  N_("June"), N_("July"), N_("August"), N_("September"), N_("October"),
  N_("November"), N_("December")};
 extern GtkWidget *main_win_scrollbar;
+extern 
 
 /*
  * -------------------
@@ -101,9 +100,9 @@ int NumTextLines (int l);
 int RepaintCalendar (GtkWidget * widget, GdkEventExpose * event);
 int RepaintLogInfo (GtkWidget *, GdkEventExpose *);
 int rapaint_zoom (GtkWidget * widget, GdkEventExpose * event);
-void HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key);
-void PointerMoved (GtkWidget * cv, GdkEventMotion * event);
-void log_repaint (GtkWidget * cv, GdkEventExpose * event);
+gboolean HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key);
+gboolean PointerMoved (GtkWidget * cv, GdkEventMotion * event);
+gboolean log_repaint (GtkWidget * cv, GdkEventExpose * event);
 void log_redrawcursor (int ol, int nl, Page * np);
 void log_redrawdetail (void);
 void DrawMonthHeader (LogLine * line, int y);
@@ -116,7 +115,7 @@ void UpdateStatusArea (void);
 void change_log (int direction);
 void create_zoom_view (GtkWidget * widget, gpointer user_data);
 void close_zoom_view (GtkWidget * widget, GtkWindow ** window);
-void handle_log_mouse_button (GtkWidget * win, GdkEventButton * event_key);
+gboolean handle_log_mouse_button (GtkWidget * win, GdkEventButton * event_key);
 Page *GetPageAtCursor (int y);
 
 /* ----------------------------------------------------------------------
@@ -124,7 +123,7 @@ Page *GetPageAtCursor (int y);
    DESCRIPTION: The pointer moved inside the window.
    ---------------------------------------------------------------------- */
 
-void
+gboolean
 PointerMoved (GtkWidget * cv, GdkEventMotion * event)
 {
    int cursory, nl;
@@ -132,7 +131,7 @@ PointerMoved (GtkWidget * cv, GdkEventMotion * event)
 
    /* Check that there is at least one log */
    if (curlog == NULL)
-      return;
+      return FALSE;
 
    cursory = event->y;
    cursor_visible = TRUE;
@@ -144,6 +143,8 @@ PointerMoved (GtkWidget * cv, GdkEventMotion * event)
       curlog->pointerpg = np;
       log_redrawdetail ();
    }
+
+   return FALSE;
 }
 
 
@@ -187,28 +188,48 @@ NumTextLines (int l)
    DESCRIPTION: User clicked on main window: open zoom view. 
    ---------------------------------------------------------------------- */
 
-void
+gboolean
 handle_log_mouse_button (GtkWidget * win, GdkEventButton *event)
 {
-  static guint32 lasttime;
-  static int clicked_before = FALSE;
 
-  if (event->type == GDK_BUTTON_PRESS && !clicked_before)
-    {
-      lasttime = event->time;
-      clicked_before = TRUE;
-      return;
-    }
-  
-  clicked_before = FALSE;
-  if (event->time - lasttime < 100 ||
-      event->time - lasttime > 200)
-    return;
+  if (event->type == GDK_2BUTTON_PRESS) {
+    if (!zoom_visible)
+      create_zoom_view (NULL, NULL);
+  }
+  else if (event->button == 1) {
+    int cursory, nl;
+    Page *np;
+    if (curlog == NULL)
+      return FALSE;
+
+     cursory = event->y;
+     cursor_visible = TRUE;
+     if ((nl = GetLineAtCursor (cursory)) != -1 && nl != curlog->pointerln)
+     {
+       np = GetPageAtCursor (cursory);
+       log_redrawcursor (curlog->pointerln, nl, np);
+       curlog->pointerln = nl;
+       curlog->pointerpg = np;
+       log_redrawdetail ();
+     }
+  } 
+
+  return FALSE;
+}
+
+gboolean
+handle_log_mouse_scroll (GtkWidget * win, GdkEventScroll *event)
+{
+  GtkAdjustment *adj;
+  adj = GTK_ADJUSTMENT (GTK_RANGE (main_win_scrollbar)->adjustment);
+
+  if (event->direction == GDK_SCROLL_UP)
+    gtk_adjustment_set_value (adj, (adj->value - 5.0));
+  else if (event->direction == GDK_SCROLL_DOWN)
+    gtk_adjustment_set_value (adj, (adj->value + 5.0));
+    
+  return FALSE;
 
-  /* If zoom is already visible ignore */
-  if (!zoom_visible)
-    create_zoom_view (NULL, NULL);
-  return;
 }
 
 /* ----------------------------------------------------------------------
@@ -216,7 +237,7 @@ handle_log_mouse_button (GtkWidget * win, GdkEventButton *event)
    DESCRIPTION: Handle all posible keyboard actions.
    ---------------------------------------------------------------------- */
 
-void
+gboolean
 HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key)
 {
   GtkAdjustment *adj;
@@ -224,7 +245,7 @@ HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key)
 
   /* Check that there is at least one log */
   if (curlog == NULL)
-    return;
+    return FALSE;
   
   adj = GTK_ADJUSTMENT (GTK_RANGE (main_win_scrollbar)->adjustment);
   key = event_key->keyval;
@@ -268,10 +289,11 @@ HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key)
       break;
       
     default:
+      return FALSE;
       break;
     };
   
-   return;
+   return TRUE;
 }
 
 /* ----------------------------------------------------------------------
@@ -385,7 +407,7 @@ ScrollUp (int howmuch)
    DESCRIPTION: Redraw screen.
    ---------------------------------------------------------------------- */
 
-void
+gboolean
 log_repaint (GtkWidget * win, GdkEventExpose * event)
 {
    static int firsttime = TRUE;
@@ -393,15 +415,27 @@ log_repaint (GtkWidget * win, GdkEventExpose * event)
    Page *pg;
    LogLine *line;
    GdkRectangle *area;
+   PangoContext *context;
+   PangoFontMetrics *metrics;
+   PangoFont *font;
 
    if (firsttime)
    {
       if (win == NULL)
-	 return;
+	 return FALSE;
       firsttime = FALSE;
+      log_layout = gtk_widget_create_pango_layout (main_win_scrollbar, "");
+      context = gdk_pango_context_get ();
+      pango_context_set_language (context, gtk_get_default_language ());
+      font = pango_context_load_font (context, cfg->fixedb);
+      metrics = pango_font_get_metrics
+                (font, pango_context_get_language (context));
       canvas = win->window;
       gc = gdk_gc_new (canvas);
-      log_line_sep = cfg->fixedb->ascent + 3;
+      log_line_sep = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics)) + 3;
+      pango_font_metrics_unref (metrics);
+      g_object_unref (G_OBJECT (context));
+      g_object_unref (G_OBJECT (font));
    }
    offset = LOG_TITLEY;
 
@@ -418,7 +452,10 @@ log_repaint (GtkWidget * win, GdkEventExpose * event)
 
    /* Check that there is at least one log */
    if (curlog == NULL)
-      return;
+   {
+      UpdateStatusArea ();	   
+      return FALSE;
+   }
 
    pg = curlog->currentpg;
    ln = curlog->firstline;
@@ -459,6 +496,8 @@ log_repaint (GtkWidget * win, GdkEventExpose * event)
 
   /* Repaint status bar */
    UpdateStatusArea ();
+
+   return TRUE;
 }
 
 void
@@ -467,10 +506,15 @@ UpdateStatusArea ()
   struct tm *tdm;
   char status_text[255];
   char *buffer;
-  const char *time_fmt = "%x"; /* an evil way to avoid warning */
+  /* Translators: Date only format, %x should well do really */
+  const char *time_fmt = _("%x"); /* an evil way to avoid warning */
 
   if (curlog == NULL)
-    return;
+  {
+      gtk_label_set_text (filename_label, "");
+      gtk_label_set_text (date_label, "");    
+      return;
+  }
 
   if (curlog->name != NULL)
     {
@@ -560,22 +604,44 @@ DrawLogLine (LogLine *line, int y)
   int num_chars, max_num_chars;
   int col_pos = 0;
   int char_width;
+  PangoRectangle logical_rect;
   
-  char_width = gdk_char_width (cfg->fixed, 'A');
-  
+  pango_layout_set_text (log_layout, "A", 1);
+  pango_layout_set_font_description (log_layout, cfg->fixed);
+  pango_layout_get_pixel_extents (log_layout, NULL, &logical_rect);
+  char_width = logical_rect.width; 
+
   /*gdk_gc_set_foreground (gc, &cfg->white); */
   gdk_gc_set_foreground (gc, &cfg->black);
-  
-  if (line->hour >= 0 && line->min >= 0 && line->sec >= 0)
-    g_snprintf (tmp, sizeof (tmp), "%02d:%02d:%02d", line->hour, line->min, line->sec);
-  else
-    strcpy (tmp, " ");
-  gdk_draw_string (canvas, cfg->fixedb, gc, LOG_COL1, y, tmp);
+
+  if (line->hour >= 0 && line->min >= 0 && line->sec >= 0) {
+	  struct tm date = {0};
+	  date.tm_mon = line->month;
+	  date.tm_year = 70 /* bogus */;
+	  date.tm_mday = line->date;
+	  date.tm_hour = line->hour;
+	  date.tm_min = line->min;
+	  date.tm_sec = line->sec;
+	  date.tm_isdst = 0;
+
+	  /* Translators: should be only the time, date could be bogus */
+	  if (strftime (tmp, sizeof (tmp), _("%X"), &date) <= 0) {
+		  /* as a backup print in 24 hours style */
+		  g_snprintf (tmp, sizeof (tmp), "%02d:%02d:%02d", line->hour, line->min, line->sec);
+	  }
+
+  } else {
+	  strcpy (tmp, " ");
+  }
+  pango_layout_set_font_description (log_layout, cfg->fixed);
+  pango_layout_set_text (log_layout, tmp, -1);
+  gdk_draw_layout (canvas, gc, LOG_COL1, y-10, log_layout);
 
   /* Print four spaces */
   col_pos = LOG_COL1 + strlen(tmp)*char_width;
   strcpy (tmp, "    ");
-  gdk_draw_string (canvas, cfg->fixedb, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
   col_pos = col_pos + 4*char_width;
 
   strcpy (tmp, " ");
@@ -585,12 +651,16 @@ DrawLogLine (LogLine *line, int y)
   	if (strlen (tmp) > user_prefs->hostname_column_width)
     		tmp[user_prefs->hostname_column_width+1] = '\0';
   }
-  gdk_draw_string (canvas, cfg->fixed, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  pango_layout_set_font_description (log_layout, cfg->fixed);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
 
   /* Print four spaces */
   col_pos = col_pos + user_prefs->hostname_column_width*char_width;
   strcpy (tmp, "    ");
-  gdk_draw_string (canvas, cfg->fixedb, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  pango_layout_set_font_description (log_layout, cfg->fixedb);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
   col_pos = col_pos + 4*char_width;
 
   strcpy (tmp, " ");
@@ -600,23 +670,29 @@ DrawLogLine (LogLine *line, int y)
   	if (strlen (tmp) > user_prefs->process_column_width)
     		tmp[user_prefs->process_column_width+1] = '\0';
   }
-  gdk_draw_string (canvas, cfg->fixed, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  pango_layout_set_font_description (log_layout, cfg->fixed);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
 
   /* Print four spaces */
   col_pos = col_pos + user_prefs->process_column_width*char_width;
   strcpy (tmp, "    ");
-  gdk_draw_string (canvas, cfg->fixedb, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  pango_layout_set_font_description (log_layout, cfg->fixedb);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
   col_pos = col_pos + 4*char_width;
 
   /* For now max string length is ignored */
   num_chars = MAX (strlen (line->message), 1023);
   tmp[1023] = '\0';
   strncpy (tmp, line->message, 1023);
-  max_num_chars = (canvas_width - 10 - col_pos) / gdk_char_width (cfg->fixed, 'A');
+  max_num_chars = (canvas_width - 10 - col_pos) / char_width;
   if (max_num_chars < num_chars)
     max_num_chars = num_chars;
   tmp[max_num_chars] = '\0';
-  gdk_draw_string (canvas, cfg->fixed, gc, col_pos, y, tmp);
+  pango_layout_set_text (log_layout, tmp, -1);
+  pango_layout_set_font_description (log_layout, cfg->fixed);
+  gdk_draw_layout (canvas, gc, col_pos, y-10, log_layout);
 }
 
 
@@ -631,13 +707,26 @@ DrawMonthHeader (LogLine * line, int y)
    char buf[100];
    int  h, centery, skip;
    GdkColor color[3];
+   PangoContext *context;
+   PangoFontMetrics *metrics;
+   PangoFont *font;
 
    color[0] = cfg->blue1;
    color[1] = cfg->blue;
    color[2] = cfg->blue3;
 
-   h = cfg->headingb->ascent - cfg->headingb->descent;
-   skip = (log_line_sep - cfg->fixed->ascent + cfg->fixed->descent);
+   context = gdk_pango_context_get ();
+   pango_context_set_language (context, gtk_get_default_language ());
+   font = pango_context_load_font (context, cfg->headingb);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   h = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics)) -
+       PANGO_PIXELS (pango_font_metrics_get_descent (metrics));
+   font = pango_context_load_font (context, cfg->fixed);	
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   skip = (log_line_sep - PANGO_PIXELS (pango_font_metrics_get_ascent (metrics))
+	   + PANGO_PIXELS (pango_font_metrics_get_descent (metrics)));
    centery = y + log_line_sep - ((2 * log_line_sep - skip - h) >> 1);
    Draw3DBox (canvas, gc, 5, y - log_line_sep + skip , canvas_width - 10, 2*log_line_sep-skip, color);
 
@@ -654,9 +743,17 @@ DrawMonthHeader (LogLine * line, int y)
    } else {
 	   g_snprintf (buf, sizeof (buf), "?%d? %d", (int) line->month, line->date);
    }
-   gdk_draw_string (canvas, cfg->headingb, gc, LOG_COL1 - 1, centery + 1, buf);
+   pango_layout_set_text (log_layout, buf, -1);
+   pango_layout_set_font_description (log_layout, cfg->headingb);
+   gdk_draw_layout (canvas, gc, LOG_COL1 - 1, centery + 1 - 12, log_layout);
    gdk_gc_set_foreground (gc, &cfg->white);
-   gdk_draw_string (canvas, cfg->headingb, gc, LOG_COL1, centery, buf);
+   pango_layout_set_text (log_layout, buf, -1);
+   pango_layout_set_font_description (log_layout, cfg->headingb);
+   gdk_draw_layout (canvas, gc, LOG_COL1, centery -12, log_layout);
+
+   g_object_unref (G_OBJECT (font));
+   pango_font_metrics_unref (metrics);
+   g_object_unref (G_OBJECT (context));
 
 }
 
@@ -739,7 +836,7 @@ InitPages ()
    numlogs = 0;
    for (i = 0; i < DEF_NUM_LOGS; i++)
      {
-       curlog = OpenLogFile (deflognames[i]);
+       curlog = OpenLogFile (user_prefs->logfile);
        if (curlog == NULL)
 	 continue;
        loglist[numlogs] = curlog;
