@@ -40,6 +40,7 @@
 int zoom_visible;
 GtkWidget *zoom_dialog;
 GtkWidget *zoom_canvas;
+PangoLayout *zoom_layout = NULL;
 
 extern ConfigData *cfg;
 extern GdkGC *gc;
@@ -50,7 +51,7 @@ extern GList *regexp_db, *descript_db;
 void close_zoom_view (GtkWidget * widget, GtkWindow ** window);
 void create_zoom_view (GtkWidget * widget, gpointer user_data);
 int match_line_in_db (LogLine *line, GList *db);
-void draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc,  \
+void draw_parbox (GdkDrawable *win, PangoFontDescription *font, GdkGC *gc,  \
 	     int x, int y, int width, char *text, int max_num_lines);
 
 /* ----------------------------------------------------------------------
@@ -64,6 +65,9 @@ create_zoom_view (GtkWidget * widget, gpointer user_data)
    GtkWidget *frame;
    GtkWidget *vbox;
    int h1, h2, height;
+   PangoContext *context;
+   PangoFontMetrics *metrics;
+   PangoFont *font;
 
    if (curlog == NULL || zoom_visible)
       return;
@@ -99,10 +103,23 @@ create_zoom_view (GtkWidget * widget, gpointer user_data)
    gtk_signal_connect (GTK_OBJECT (zoom_canvas), "expose_event",
 		       (GtkSignalFunc) repaint_zoom, zoom_canvas);
 
-   h1 = cfg->headingb->descent + cfg->headingb->ascent;
-   h2 = cfg->fixedb->descent + cfg->fixedb->ascent+2;
+   zoom_layout = gtk_widget_create_pango_layout (zoom_canvas, "");
+   context = gdk_pango_context_get ();
+   pango_context_set_language (context, gtk_get_default_language ()); 
+   font = pango_context_load_font (context, cfg->headingb);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   h1 = PANGO_PIXELS (pango_font_metrics_get_descent (metrics)) +
+	PANGO_PIXELS (pango_font_metrics_get_ascent (metrics));
+   font = pango_context_load_font (context, cfg->fixedb);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   h2 = PANGO_PIXELS (pango_font_metrics_get_descent (metrics)) +
+	PANGO_PIXELS (pango_font_metrics_get_ascent (metrics))+2;
    height = h1*2+14*h2;
-   gtk_drawing_area_size (GTK_DRAWING_AREA (zoom_canvas), ZOOM_WIDTH, height);
+   GTK_WIDGET (zoom_canvas)->requisition.height = height;
+   GTK_WIDGET (zoom_canvas)->requisition.width = ZOOM_WIDTH;
+   gtk_widget_queue_resize (GTK_WIDGET (zoom_canvas));
    gtk_widget_set_events (zoom_canvas, GDK_EXPOSURE_MASK);
    /* gtk_widget_set_style (zoom_canvas, cfg->black_bg_style); */
    gtk_widget_set_style (zoom_canvas, cfg->white_bg_style);
@@ -111,8 +128,12 @@ create_zoom_view (GtkWidget * widget, gpointer user_data)
    gtk_widget_realize (zoom_canvas);
 
    gtk_widget_show (zoom_dialog);
+   gtk_widget_queue_draw (zoom_canvas);
 
    zoom_visible = TRUE;
+   g_object_unref (G_OBJECT (context));
+   pango_font_metrics_unref (metrics);
+   g_object_unref (G_OBJECT (font));
 }
 
 /* ----------------------------------------------------------------------
@@ -125,10 +146,16 @@ repaint_zoom (GtkWidget * widget, GdkEventExpose * event)
 {
    static GdkDrawable *canvas;
    char buffer[1024];
-   int x, y, h, w, win_width;
+   int x , y , h, w, win_width;
    int win_height;
    GdkRectangle *area;
    LogLine *line;
+   struct tm date = {0};
+   int af, df, afdb, dfdb, ah, dh;
+   PangoContext *context;
+   PangoFontMetrics *metrics;
+   PangoFont *font;
+   PangoRectangle logical_rect;
 
    canvas = zoom_canvas->window;
    win_width = zoom_canvas->allocation.width;
@@ -141,10 +168,20 @@ repaint_zoom (GtkWidget * widget, GdkEventExpose * event)
        area = &event->area;
 
    /* Draw title */
-   h = cfg->headingb->descent + cfg->headingb->ascent;
-   w = gdk_string_measure (cfg->fixedb, "X");
+   context = gdk_pango_context_get ();
+   pango_context_set_language (context, gtk_get_default_language ());
+   font = pango_context_load_font (context, cfg->headingb);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   ah = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics));
+   dh = PANGO_PIXELS (pango_font_metrics_get_descent (metrics));
+   h = dh + ah;
+   pango_layout_set_text (zoom_layout, "X", -1);
+   pango_layout_set_font_description (zoom_layout, cfg->fixedb);
+   pango_layout_get_pixel_extents (zoom_layout, NULL, &logical_rect);
+   w = logical_rect.width;
    x = 5;
-   y = cfg->headingb->ascent + 6;
+   y = ah + 6;
    gdk_gc_set_foreground (gc, &cfg->blue);
    gdk_draw_rectangle (canvas, gc, TRUE, 3, 3, win_width - 6, h+6);
    gdk_gc_set_foreground (gc, &cfg->white);
@@ -154,68 +191,104 @@ repaint_zoom (GtkWidget * widget, GdkEventExpose * event)
    else
      g_snprintf (buffer, sizeof (buffer),
 		 _("Log line detail for <No log loaded>"));
-   gdk_draw_string (canvas, cfg->headingb, gc, x, y, buffer);
-   y += 9 + cfg->headingb->descent + cfg->fixedb->ascent;
-
+   pango_layout_set_text (zoom_layout, buffer, -1);
+   pango_layout_set_font_description (zoom_layout, cfg->headingb);
+   gdk_draw_layout (canvas, gc, x, y-12, zoom_layout);
+   font = pango_context_load_font (context, cfg->fixedb);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   afdb = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics));
+   y += 9 + dh + afdb;
 
    /* Draw Info */
    gdk_gc_set_foreground (gc, &cfg->black);
    /* gdk_gc_set_foreground (gc, &cfg->white); */
 
-   h = cfg->fixedb->descent + cfg->fixedb->ascent+2;
+   h = dfdb + afdb+2;
 
    /* Draw bg. rectangles */
+   font = pango_context_load_font (context, cfg->fixed);
+   metrics = pango_font_get_metrics
+	     (font, pango_context_get_language (context));
+   af = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics));
+   df = PANGO_PIXELS (pango_font_metrics_get_descent (metrics));
+
    gdk_gc_set_foreground (gc, &cfg->blue3);
    gdk_draw_rectangle (canvas, gc, TRUE, 15*w+6, 
-		       y - cfg->fixed->ascent-3, 
+		       y - af-3, 
 		       win_width - 9 - 15*w, 
-		       win_height - (y - cfg->fixed->ascent) );
+		       win_height - (y - af) );
    gdk_gc_set_foreground (gc, &cfg->gray75);
    gdk_draw_rectangle (canvas, gc, TRUE, 3, 
-		       y-cfg->fixed->ascent-3, 
-		       15*w, win_height - (y - cfg->fixed->ascent) );
+		       y-af-3, 
+		       15*w, win_height - (y - af) );
 
    gdk_gc_set_foreground (gc, &cfg->black);
-   gdk_draw_string (canvas, cfg->fixedb, gc, x, y, _("Date:"));
+   pango_layout_set_text (zoom_layout, _("Date:"), -1);
+   pango_layout_set_font_description (zoom_layout, cfg->fixedb);
+   gdk_draw_layout (canvas, gc, x, y, zoom_layout);
    y += h;
-   gdk_draw_string (canvas, cfg->fixedb, gc, x, y, _("Process:"));
+   pango_layout_set_text (zoom_layout, _("Process:"), -1);
+   gdk_draw_layout (canvas, gc, x, y, zoom_layout);
    y += h;
-   gdk_draw_string (canvas, cfg->fixedb, gc, x, y, _("Message:"));
+   pango_layout_set_text (zoom_layout, _("Message:"), -1);
+   gdk_draw_layout (canvas, gc, x, y, zoom_layout);
    y += 4*h;
-   gdk_draw_string (canvas, cfg->fixedb, gc, x, y, _("Description:"));
-
-   gdk_gc_set_font (gc, cfg->fixed);
+   pango_layout_set_text (zoom_layout, _("Description:"), -1);
+   gdk_draw_layout (canvas, gc, x, y-8, zoom_layout);
 
    /* Check that there is at least one log */
    if (curlog == NULL)
       return -1;
 
    /* Draw line info */
-   h = cfg->headingb->descent + cfg->headingb->ascent;
-   y = cfg->headingb->ascent + 6;
-   y += 9 + cfg->headingb->descent + cfg->fixedb->ascent;
+   h = dh + ah;
+   y = ah + 6;
+   y += 9 + dh + afdb;
    x = 15*w + 6 + 2;
-   h = cfg->fixedb->descent + cfg->fixedb->ascent+2;
+   h = dfdb + afdb;
 
    line = &(curlog->pointerpg->line[curlog->pointerln]);
 
-   g_snprintf (buffer, sizeof (buffer),
-	       "%s %d %02d:%02d:%02d", _(month[(int)line->month]), 
-	       (int) line->date, line->hour, line->min, line->sec);
-   gdk_draw_string (canvas, cfg->fixed, gc, x, y, buffer);
+   date.tm_mon = line->month;
+   date.tm_year = 70 /* bogus */;
+   date.tm_mday = line->date;
+   date.tm_hour = line->hour;
+   date.tm_min = line->min;
+   date.tm_sec = line->sec;
+   date.tm_isdst = 0;
+
+   /* Translators: do not include year, it would be bogus */
+   if (strftime (buffer, sizeof (buffer), _("%B %e %X"), &date) <= 0) {
+	   /* as a backup print in US style */
+	   g_snprintf (buffer, sizeof (buffer),
+		       "%s %d %02d:%02d:%02d",
+		       _(month[(int)line->month]), 
+		       (int) line->date,
+		       (int) line->hour,
+		       (int) line->min,
+		       (int) line->sec);
+   }
+   pango_layout_set_font_description (zoom_layout, cfg->fixed);
+   pango_layout_set_text (zoom_layout, buffer, -1);
+   gdk_draw_layout (canvas, gc, x, y, zoom_layout);
    y += h;
    g_snprintf (buffer, sizeof (buffer), "%s ", line->process);
-   gdk_draw_string (canvas, cfg->fixed, gc, x, y, buffer);
+   pango_layout_set_text (zoom_layout, buffer, -1);
+   gdk_draw_layout (canvas, gc, x, y+2, zoom_layout);
    y += h;
-   draw_parbox (canvas, cfg->fixed, gc, x, y, 380, line->message, 4);
+   draw_parbox (canvas, cfg->fixed, gc, x, y+4, 380, line->message, 4);
    y += 4*h;
    if (match_line_in_db (line, regexp_db))
      {
        if (find_tag_in_db (line, descript_db))
-	 draw_parbox (canvas, cfg->fixed, gc, x, y, 380,
+	 draw_parbox (canvas, cfg->fixed, gc, x, y+4, 380,
 			  line->description->description, 8);
      }
 
+   g_object_unref (G_OBJECT (context));
+   pango_font_metrics_unref (metrics);
+   g_object_unref (G_OBJECT (font));
    return TRUE;
 }
 
@@ -227,15 +300,26 @@ repaint_zoom (GtkWidget * widget, GdkEventExpose * event)
    ---------------------------------------------------------------------- */
 
 void
-draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc, 
+draw_parbox (GdkDrawable *win, PangoFontDescription *font, GdkGC *gc, 
 	     int x, int y, int width, char *text, int max_num_lines)
 {
    char *p1, *p2, *p3, c;
    char *buffer;
    int  ypos, done;
+   PangoContext *context;
+   PangoFontMetrics *metrics;
+   PangoRectangle logical_rect;
+   PangoFont *pfont;
 
    ypos = y;
 
+   context = gdk_pango_context_get ();
+   pango_context_set_language (context, gtk_get_default_language ());
+   pfont = pango_context_load_font (context, font);
+   metrics = pango_font_get_metrics
+	     (pfont, pango_context_get_language (context));
+   pango_layout_set_font_description (zoom_layout, font);
+
    /* Copy string and modify our buffer string */
    buffer = g_strdup(text);
 
@@ -244,22 +328,28 @@ draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc,
    while (!done)
      {
        c = *p1; *p1 = 0;
-       while ( gdk_string_measure (font, p2) < width && c != 0)
+       pango_layout_set_text (zoom_layout, p2, -1);
+       pango_layout_get_pixel_extents (zoom_layout, NULL, &logical_rect);
+       while ( logical_rect.width < width && c != 0)
          {
 	   *p1 = c;
 	   p1++;
 	   c = *p1;
 	   *p1 = 0;
+	   pango_layout_set_text (zoom_layout, p2, -1);
+	   pango_layout_get_pixel_extents (zoom_layout, NULL, &logical_rect);
          }
        switch (c)
          {
          case ' ':
 	   p1++;
-	   gdk_draw_string (win, font, gc, x, ypos, p2);
+	   pango_layout_set_text (zoom_layout, p2, -1);
+	   gdk_draw_layout (win, gc, x, ypos, zoom_layout);
 	   break;
          case '\0':
 	   done = TRUE;
-	   gdk_draw_string (win, font, gc, x, ypos, p2);
+	   pango_layout_set_text (zoom_layout, p2, -1);
+	   gdk_draw_layout (win, gc, x, ypos, zoom_layout);
 	   break;
          default:
 	   p3 = p1;
@@ -267,7 +357,8 @@ draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc,
 	     p1--;
 	   if (p1 == p2)
 	     {
-               gdk_draw_string (win, font, gc, x, ypos, p2);
+	       pango_layout_set_text (zoom_layout, p2, -1);
+	       gdk_draw_layout (win, gc, x, ypos, zoom_layout);
                *p3 = c;
                p1 = p3;
                break;
@@ -277,14 +368,19 @@ draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc,
                *p3 = c;
                *p1 = 0;
                p1++;
-               gdk_draw_string (win, font, gc, x, ypos, p2);
+	       pango_layout_set_text (zoom_layout, p2, -1);
+	       gdk_draw_layout (win, gc, x, ypos, zoom_layout);
 	     }
 	   break;
          }
-       ypos += font->descent + font->ascent;
+       ypos += PANGO_PIXELS (pango_font_metrics_get_descent (metrics)) +
+	       PANGO_PIXELS (pango_font_metrics_get_ascent (metrics)); 
        p2 = p1;
      }
    
+   g_object_unref (G_OBJECT (pfont));
+   pango_font_metrics_unref (metrics);
+   g_object_unref (G_OBJECT (context));
    g_free (buffer);
 
 }
@@ -298,8 +394,11 @@ draw_parbox (GdkDrawable *win, GdkFont *font, GdkGC *gc,
 void
 close_zoom_view (GtkWidget * widget, GtkWindow ** window)
 {
-   if (zoom_visible)
+   if (zoom_visible) {
       gtk_widget_hide (zoom_dialog);
+      if (G_IS_OBJECT (zoom_layout))
+         g_object_unref (G_OBJECT (zoom_layout));
+   }
    zoom_dialog = NULL;
    zoom_visible = FALSE;
 }
